"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const webpack = require("webpack");
const ExtractTextPlugin = require("extract-text-webpack-plugin");
const CompressionPlugin = require("compression-webpack-plugin");
const glob = require("glob");
const path = require("path");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const CleanWebpackPlugin = require('clean-webpack-plugin');
const OptimizeCssAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const ClosureCompilerPlugin = require('webpack-closure-compiler');
const PurifyCSSPlugin = require('purifycss-webpack');
const HardSourceWebpackPlugin = require('hard-source-webpack-plugin');
function setDefaults(options) {
    //ENTRY DEFAULTS
    if (typeof options.entry === 'undefined') {
        options.entry = {
            'client': path.resolve(options.srcPath, "index.tsx")
        };
    }
    //ENVIRONMENT DEFAULTS
    if (typeof options.environmentVariables === 'undefined') {
        options.environmentVariables = {
            NODE_ENV: 'production',
            DEBUG: false
        };
    }
    //CHUNKING DEFAULTS
    if (((typeof options.chunkingOptions === 'undefined') && (options.chunkingOptions = {})) || options.chunkingOptions) {
        if (typeof options.chunkingOptions.commonsChunkOptions === 'undefined') {
            options.chunkingOptions.commonsChunkOptions = {
                name: "common",
                filename: "common.js",
                minChunks: 2
            };
        }
        if (typeof options.chunkingOptions.aggressiveSplittingOptions === 'undefined') {
            options.chunkingOptions.aggressiveSplittingOptions = {
                minSize: 100000,
                maxSize: 150000,
            };
        }
    }
    //JS OPTIMIZATION DEFAULTS
    if (typeof options.uglifyJsOptions === 'undefined') {
        options.uglifyJsOptions = {
            beautify: false,
            mangle: {
                screw_ie8: true,
                keep_fnames: true
            },
            compress: {
                warnings: false,
                screw_ie8: true,
                conditionals: true,
                unused: true,
                comparisons: true,
                sequences: true,
                dead_code: true,
                evaluate: true,
                if_return: true,
                join_vars: true
            },
            comments: false,
            sourceMap: false
        };
    }
    if (typeof options.closureCompilerOptions === 'undefined') {
        options.closureCompilerOptions = {
            jsCompiler: true,
            compiler: {
                language_in: 'ECMASCRIPT5',
                language_out: 'ECMASCRIPT5',
                compilation_level: 'ADVANCED',
                warningLevel: 'QUIET'
            },
        };
    }
    //CSS OPTIMIZATION DEFAULTS
    if (((typeof options.cssOptimizationOptions === 'undefined') && (options.cssOptimizationOptions = {})) || options.cssOptimizationOptions) {
        if (typeof options.cssOptimizationOptions.extractTextOptions === 'undefined') {
            options.cssOptimizationOptions.extractTextOptions = "styles.css";
        }
        if (typeof options.cssOptimizationOptions.purifyCSSOptions === 'undefined') {
            options.cssOptimizationOptions.purifyCSSOptions = {
                paths: glob.sync(path.join(options.srcPath, '**/*.tsx')),
            };
        }
        if (typeof options.cssOptimizationOptions.optimizeCSSAssetsOptions === 'undefined') {
            options.cssOptimizationOptions.optimizeCSSAssetsOptions = {
                assetNameRegExp: /\.css$/g,
                cssProcessor: require('cssnano'),
                cssProcessorOptions: { discardComments: { removeAll: true } },
                canPrint: true
            };
        }
    }
    //COMPRESSION DEFAULTS
    if (typeof options.compressionOptions === 'undefined') {
        options.compressionOptions = {
            asset: "[path].gz[query]",
            algorithm: "gzip",
            test: /\.(js|html|css)$/,
            threshold: 10240,
            minRatio: 0.8
        };
    }
}
function configFactory(options) {
    setDefaults(options);
    //PLUGIN DEFAULTS
    let plugins = [
        new CleanWebpackPlugin([path.basename(options.outputPath)], { root: path.dirname(options.outputPath) }),
        new HtmlWebpackPlugin({
            template: options.htmlIndexPath
        }),
        new HardSourceWebpackPlugin()
    ];
    //RULE DEFAULTS
    let rules = [
        {
            test: /\.(tsx|ts)?$/,
            use: {
                loader: 'awesome-typescript-loader',
                options: {
                    configFileName: options.tsconfigPath ? options.tsconfigPath : 'tsconfig.json'
                }
            }
        },
        //makes sure that we get the sourcemaps from any libraries
        {
            test: /\.js$/,
            use: [{
                    loader: 'source-map-loader'
                }],
        },
        //images
        {
            test: /\.(woff|woff2|ttf|eot|jpe?g|png|gif)$/,
            use: {
                loader: "url-loader",
                options: {
                    limit: 8192
                }
            }
        },
    ];
    /*******************************************
    Alter the rules and plugins based on the webpack options
    *******************************************/
    //environmental settings
    plugins.push(new webpack.EnvironmentPlugin(options.environmentVariables));
    //javascript optimizers
    if (options.uglifyJsOptions) {
        plugins.push(new webpack.optimize.UglifyJsPlugin());
    }
    if (options.closureCompilerOptions) {
        plugins.push(new ClosureCompilerPlugin(options.closureCompilerOptions));
    }
    //css optimizations
    if (options.cssOptimizationOptions) {
        plugins.push(new ExtractTextPlugin(options.cssOptimizationOptions.extractTextOptions), new PurifyCSSPlugin(options.cssOptimizationOptions.purifyCSSOptions), new OptimizeCssAssetsPlugin(options.cssOptimizationOptions.optimizeCSSAssetsOptions));
        rules.push({
            test: /\.scss$/,
            use: ExtractTextPlugin.extract({
                fallback: "style-loader",
                use: [
                    {
                        loader: "css-loader",
                        options: {
                            sourceMap: options.sourceMaps === true
                        }
                    },
                    {
                        loader: "sass-loader",
                        options: {
                            sourceMap: options.sourceMaps === true
                        }
                    }
                ]
            })
        }, {
            test: /\.css$/,
            use: ExtractTextPlugin.extract({
                fallback: "style-loader",
                use: [
                    {
                        loader: "css-loader",
                        options: {
                            sourceMap: options.sourceMaps === true
                        }
                    }
                ]
            })
        });
    }
    else {
        rules.push({
            test: /\.scss$/,
            use: [
                {
                    loader: "style-loader"
                },
                {
                    loader: "css-loader",
                    options: {
                        sourceMap: options.sourceMaps === true
                    }
                },
                {
                    loader: "sass-loader",
                    options: {
                        sourceMap: options.sourceMaps === true
                    }
                }
            ]
        }, {
            test: /\.css$/,
            use: [
                {
                    loader: "style-loader"
                },
                {
                    loader: "css-loader",
                    options: {
                        sourceMap: options.sourceMaps === true
                    }
                }
            ]
        });
    }
    if (options.compressionOptions) {
        plugins.push(new CompressionPlugin(options.compressionOptions));
    }
    if (options.chunkingOptions) {
        plugins.push(new webpack.optimize.CommonsChunkPlugin(options.chunkingOptions.commonsChunkOptions));
    }
    /*******************************************
    Construct the configuration option
    *******************************************/
    return {
        entry: options.entry,
        output: {
            path: options.outputPath,
            filename: '[id].bundle.js',
            chunkFilename: '[id].chunk.js',
            publicPath: '/'
        },
        devtool: options.sourceMaps ? 'source-map' : false,
        resolve: {
            extensions: ['.ts', '.js', '.scss', '.tsx', '.jsx']
        },
        module: {
            rules: rules
        },
        plugins: plugins
    };
}
exports.default = configFactory;
